import numpy as np
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
from math import atan, sin, exp, floor


class ExtendBHCurve:

    @staticmethod
    def extend_bhcurve_arctan(material_dictionary):
        # mu = 1/alpha+beta*|H|
        mu_0 = 4 * np.pi * 1e-7
        H = material_dictionary['field_strength'][0]
        B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['polarization'][0]],
                                   [h * mu_0 for h in material_dictionary['field_strength'][0]])]

        B_sat = B[-1]
        mu_d = (B[0])/(H[0])/mu_0    # relative permeability for H=0: (B[0] - 0.0)/(H[0]-0.0)/mu_0
        alpha = 1.2  # 2.0 * B_sat / np.pi
        print("alpha = ", alpha)
        beta = 1000  # (2.0 * B_sat) / np.pi * mu_0 * mu_d
        print("beta = ", beta)
        h_ext = np.arange(H[-1]*1.5, 500000, 1000).tolist()  # [50000, 70000, 100000, 150000, 200000, 300000, 400000, 500000]
        H_extrapolated = np.concatenate((H, h_ext), axis=None)
        # print('H extrapolated = ', H_extrapolated)
        B_extrapolated = B
        for num in h_ext:
            B_extrapolated = np.concatenate((B_extrapolated, atan(num / beta) * alpha + mu_0 * num), axis=None)
        # B_extrapolated = atan(H_extrapolated / beta) * alpha + 4 * pi * 10 ^ -7 * H_extrapolated
        print('B estrapolated = ', B_extrapolated)
        # % B = atan(H / 300) * 1.12 + 4 * pi * 10 ^ -7 * H;
        # % B = [B_orig, [1.7710, 1.8750, 1.9620, 2.1000, 2.3100, 2.5200, 2.5950, 2.7000, 2.7900, 2.9400, 3.0700, 3.1960]];
        # H_neu = H;

        # B_extrapolated = B.extend([B_extrapolated[-7:-1]])
        # print('B estrapolated = ', B_extrapolated)
        plt.plot(H, B, "ro", H, B, "b-")
        plt.show()

        plt.plot(H, B, "ro", H_extrapolated[0:len(H)+5], B_extrapolated[0:len(H)+5], "b-")
        plt.show()

        plt.plot(H_extrapolated, B_extrapolated, "ro", H, B, "b-")
        x = np.linspace(0, H_extrapolated[-1], 2*H_extrapolated[-1])
        y = [atan(z / beta) * alpha + mu_0 * z for z in x]

        plt.plot(x, y, "g-")
        plt.show()

        return H_extrapolated, B_extrapolated

    @staticmethod
    def interpolate_bhcurve_arctan(material_dictionary):
        # mu = 1/alpha+beta*|H|
        mu_0 = 4 * np.pi * 1e-7
        H = material_dictionary['field_strength'][0]
        B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['polarization'][0]],
                                   [h * mu_0 for h in material_dictionary['field_strength'][0]])]

        def test_func(x, a, b):
            return [atan(z / b) * a + mu_0 * z for z in x]

        params, params_covariance = curve_fit(test_func, H[-20:-1], B[-20:-1], p0=[1, 1000])

        print(params)

        h_ext = np.arange(H[-1] * 1.5, 500000, 1000).tolist()  # [50000, 70000, 100000, 150000, 200000, 300000, 400000, 500000]
        H_extrapolated = np.concatenate((H, h_ext), axis=None)
        # print('H extrapolated = ', H_extrapolated)
        B_extrapolated = B
        #for num in h_ext:
        #    B_extrapolated = np.concatenate((B_extrapolated, test_func(num, params[0], params[1])), axis=None)
        B_extrapolated = np.concatenate((B_extrapolated, test_func(h_ext, params[0], params[1])), axis=None)
        # B_extrapolated = atan(H_extrapolated / beta) * alpha + 4 * pi * 10 ^ -7 * H_extrapolated
        print('B estrapolated = ', B_extrapolated)
        # % B = atan(H / 300) * 1.12 + 4 * pi * 10 ^ -7 * H;
        # % B = [B_orig, [1.7710, 1.8750, 1.9620, 2.1000, 2.3100, 2.5200, 2.5950, 2.7000, 2.7900, 2.9400, 3.0700, 3.1960]];
        # H_neu = H;

        # B_extrapolated = B.extend([B_extrapolated[-7:-1]])
        # print('B estrapolated = ', B_extrapolated)
        plt.plot(H, B, "ro", H, B, "b-")
        plt.show()

        plt.plot(H, B, "ro", H_extrapolated[0:len(H) + 5], B_extrapolated[0:len(H) + 5], "b-")
        plt.show()

        plt.plot(H_extrapolated, B_extrapolated, "ro", H, B, "b-")
        x = np.linspace(0, H_extrapolated[-1], 2 * H_extrapolated[-1])
        y = test_func(x, params[0], params[1])

        plt.plot(x, y, "g-")
        plt.show()

        return H_extrapolated, B_extrapolated

    @staticmethod
    def interpolate_bhcurve_mur(material_dictionary):
        # mu = 1/alpha+beta*|H|
        mu_0 = 4 * np.pi * 1e-7
        H = material_dictionary['field_strength'][0]
        B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['polarization'][0]],
                                   [h * mu_0 for h in material_dictionary['field_strength'][0]])]

        h_ext = np.arange(H[-1] * 1.5, 500000, 1000).tolist()
        H_extrapolated = np.concatenate((H, h_ext), axis=None)
        # print('H extrapolated = ', H_extrapolated)

        mu_r = np.array([(B[i]) / (H[i]) / mu_0 for i in range(0, len(H))])
        # mu_rel = np.array([(B[i]-B[i-1]) / (H[i]-H[i-1]) / mu_0 for i in range(1, len(H))])
        # mur = 1 + (mu ) / (1 + beta * B + pow(B,n))

        plt.plot(B, mu_r, "bo")
        # plt.plot(B[0:-1], mu_rel, "bo")
        plt.show()

        plt.plot(H, mu_r, "go")
        # plt.plot(H[0:-1], mu_rel, "go")
        plt.show()

        def test_exp(x, a, b):
            return [a * exp(z * b) + 1 for z in x]

        def test_arctan(x, a, b, c):
            return [a * atan(z * b + c) + np.pi/2.0 * a + 1 for z in x]

        # param_bounds_exp = ([6000, -0.01], [np.inf, 0])
        # params_exp, params_covariance_exp = curve_fit(test_exp, H[-3:-1], mu_r[-3:-1], bounds=param_bounds_exp)
        #
        # print(params_exp)
        # # params_exp = [10000, -1.0/800.0]
        # mu_r_exp = mu_r
        # mu_r_exp = np.concatenate((mu_r_exp, test_exp(h_ext, params_exp[0], params_exp[1])), axis=None)
        # plt.plot(H, mu_r, "ro", H_extrapolated[0:len(H) + 5], mu_r_exp[0:len(H) + 5], "b-")
        # plt.show()
        #
        # x_exp = np.linspace(0, H_extrapolated[-1], 2 * H_extrapolated[-1])
        # y_exp = test_exp(x_exp, params_exp[0], params_exp[1])
        #
        # plt.plot(H[-13:-1], mu_r[-13:-1], "ro", H, mu_r, "b-")
        # plt.plot(x_exp[0:6000], y_exp[0:6000], "g-")
        # plt.show()
        #
        # plt.plot(H, mu_r, "ro", H, mu_r, "b-")
        # plt.plot(x_exp, y_exp, "g-")
        # plt.show()

        param_bounds_arctan = ([0, -np.inf, -100], [np.inf, 0, 100])
        params_arctan, params_covariance_arctan = curve_fit(test_arctan, H[-13:-1], mu_r[-13:-1], bounds=param_bounds_arctan)

        print(params_arctan)
        # params_arctan = [0.1, 10]

        mu_r_arctan = mu_r
        mu_r_arctan = np.concatenate((mu_r_arctan, test_arctan(h_ext, params_arctan[0], params_arctan[1], params_arctan[2])), axis=None)
        plt.plot(H, mu_r, "ro", H_extrapolated[0:len(H) + 5], mu_r_arctan[0:len(H) + 5], "b-")
        plt.show()

        x_arctan = np.linspace(0, H_extrapolated[-1], 2 * H_extrapolated[-1])
        y_arctan = test_arctan(x_arctan, params_arctan[0], params_arctan[1], params_arctan[2])

        plt.plot(H[-13:-1], mu_r[-13:-1], "ro", H, mu_r, "b-")
        plt.plot(x_arctan[0:10000], y_arctan[0:10000], "g-")
        #plt.plot(x_arctan, y_arctan, "g-")
        plt.show()

        print('max_value = ', test_arctan([1e10], params_arctan[0], params_arctan[1], params_arctan[2]))

        plt.plot(H_extrapolated, mu_r_arctan, "ro", H, mu_r, "b-")
        plt.plot(x_arctan, y_arctan, "g-")
        plt.show()

        B_extrapolated = B
        n = len(mu_r)
        for i in range(len(h_ext)):
            num = h_ext[i]
            b_value = mu_r_arctan[n + i] * mu_0 * H_extrapolated[n+i]
            B_extrapolated = np.concatenate((B_extrapolated, b_value), axis=None)

        plt.plot(H, B, "ro", H_extrapolated[0:len(H) + 5], B_extrapolated[0:len(H) + 5], "b-")
        plt.show()

        plt.plot(H_extrapolated, B_extrapolated, "ro", H, B, "b-")
        x = np.linspace(0, H_extrapolated[-1], 2 * H_extrapolated[-1])
        y = test_arctan(x, params_arctan[0], params_arctan[1], params_arctan[2])

        plt.plot(x, y, "g-")
        plt.show()

        return H_extrapolated, B_extrapolated

    @staticmethod
    def interpolate_bhcurve_murel(material_dictionary, type):
        # mu = 1/alpha+beta*|H|
        mu_0 = 4 * np.pi * 1e-7
        H = material_dictionary['field_strength'][0]
        B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['polarization'][0]],
                                   [h * mu_0 for h in material_dictionary['field_strength'][0]])]

        h_ext = np.arange(H[-1], 500000, 1000).tolist()[1:-1]

        H_extrapolated = np.concatenate((H, h_ext), axis=None)
        # print('H extrapolated = ', H_extrapolated)

        mu_rel = np.array([(B[i]-B[i-1]) / (H[i]-H[i-1]) / mu_0 for i in range(1, len(H))])
        # mur = 1 + (mu ) / (1 + beta * B + pow(B,n))

        plt.plot(B[0:-1], mu_rel, "bo")
        plt.title('B - mur_relativ')
        plt.xlabel('B')
        plt.ylabel('mur_rel')
        plt.show()

        plt.plot(H[0:-1], mu_rel, "go")
        plt.title('H - mur_relativ')
        plt.xlabel('H')
        plt.ylabel('mur_rel')
        plt.show()

        def fitting_function(x, a, b, c):
            y_value = []
            if type == 'arctan':
                # y_value = [a * atan(z * b + c) + np.pi / 2.0 * a + 1 for z in x]
                y_value = [max(a * atan(z * b + c) - a * atan(500000 * b + c) + 1, 1.0) for z in x]
            elif type == 'exp':
                y_value = [a * exp(z * b + c) + 1 for z in x]
            elif type == 'logistic':
                y_value = [1.0 / (1 + exp(-z * b) * (1.0 / c - 1)) for z in x]

            return y_value

        param_bounds = []
        if type == 'arctan':
            param_bounds = ([0, -np.inf, -100], [np.inf, 0, 100])
        elif type == 'exp':
            param_bounds = ([0, -np.inf, -100], [np.inf, 0, 100])  # ([-np.inf, -np.inf, -np.inf], [np.inf, np.inf, np.inf])  #
        elif type == 'logistic':
            param_bounds = ([0, 0, 1000], [1, 1e-5, 20000])

        fit_params, fit_covariance = curve_fit(fitting_function, H[-13:-1], mu_rel[-13:-1], bounds=param_bounds)

        print(fit_params)
        # params_arctan = [0.1, 10]

        mu_r_arctan = mu_rel
        mu_r_arctan = np.concatenate(
            (mu_r_arctan, fitting_function(h_ext, fit_params[0], fit_params[1], fit_params[2])), axis=None)
        plt.plot(H[0:-1], mu_rel, "ro", H_extrapolated[0:len(H) + 5], mu_r_arctan[0:len(H) + 5], "b-")
        plt.title('H - mur_relativ')
        plt.xlabel('H')
        plt.ylabel('mur_rel')
        plt.show()

        x_arctan = np.linspace(0, H_extrapolated[-1], floor(H_extrapolated[-1]))
        y_arctan = fitting_function(x_arctan, fit_params[0], fit_params[1], fit_params[2])

        plt.plot(H[-13:-1], mu_rel[-13:-1], "ro", H[0:-1], mu_rel, "b-")
        plt.plot(x_arctan[0:10000], y_arctan[0:10000], "g-")
        # plt.plot(x_arctan, y_arctan, "g-")
        plt.title('H - mur_relativ')
        plt.xlabel('H')
        plt.ylabel('mur_rel')
        plt.show()

        print('max_value = ', fitting_function([H_extrapolated[-1]], fit_params[0], fit_params[1], fit_params[2]))
        print('inf_value = ', fitting_function([1e10], fit_params[0], fit_params[1], fit_params[2]))

        plt.plot(H_extrapolated[0:-1], mu_r_arctan, "ro", H[0:-1], mu_rel, "b-")
        plt.plot(x_arctan, y_arctan, "g-")
        plt.title('H - mur_relativ')
        plt.xlabel('H')
        plt.ylabel('mur_rel')
        plt.show()

        B_extrapolated = B
        n = len(mu_rel)
        for i in range(len(h_ext)):
            num = h_ext[i]
            b_value = mu_r_arctan[n + i] * mu_0 * (H_extrapolated[n + i +1] - H_extrapolated[n + i]) + B_extrapolated[n + i]
            B_extrapolated = np.concatenate((B_extrapolated, b_value), axis=None)

        plt.plot(H, B, "ro", H_extrapolated[0:len(H) + 5], B_extrapolated[0:len(H) + 5], "b-")
        plt.title('BH-Curve')
        plt.xlabel('H')
        plt.ylabel('B')
        plt.show()

        plt.plot(H_extrapolated, B_extrapolated, "ro", H, B, "b-")
        #x = np.linspace(0, H_extrapolated[-1], 2 * H_extrapolated[-1])
        #y = test_arctan(x, params_arctan[0], params_arctan[1], params_arctan[2])

        #plt.plot(x, y, "g-")
        plt.title('BH-Curve')
        plt.xlabel('H')
        plt.ylabel('B')
        plt.show()

        print('mu_rel', mu_rel)
        print('mu_r_arctan', mu_r_arctan)

        return H_extrapolated, B_extrapolated
