#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 6189 $
$LastChangedDate:: 2022-09-28 11:12:19 +0200#$

This file is part of SyMSpace/ComponentSpace.

Copyright (c) 2022, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
from componentspace.geometry_component import GeometryComponent, FreeCADComponent, CreoComponent
import componentspace.osutils.fileutils as fu
from symspace.wrapper.symwrapper import SymDirectory


class FreeCADSectorGeometry(FreeCADComponent):

    def convertFunction(self):
        """
        Convert container to SymPythonFunction
        """

        ''' Call convertFunction from class GeometryComponent '''
        super(FreeCADComponent, self).convertFunction()

        ''' Copy _3DModel '''
        sdir = self.sym_mdl.getObject('FreeCAD.Files.3DModel').getData()
        dir_mdl = os.path.join(self.sym_mdl.getAllFilesDirectory(), '_3DModel')
        fu.copyTree(sdir, dir_mdl)

        ''' Add 3DModel to component '''
        model3D = SymDirectory('3DModel')
        self.sym_mdl.getObject('Files').addParam(model3D)
        model3D.setData(dir_mdl)

        par_part = self.sym_mdl.getObject('Project.Stator.Slot.Part')
        par_part.setData(os.path.join(dir_mdl, os.path.basename(par_part.getData())))
        for par in self.sym_mdl.getObject('Project.Stator.Slot.Settings').getParameters('SymFile'):
            par.setData(os.path.join(dir_mdl, os.path.basename(par.getData())))

        self.sym_mdl.regenerate()


class CreoSectorGeometry(CreoComponent):

    def convertFunction(self):

        super(CreoComponent, self).convertFunction()

        ''' Copy _3DModel '''
        sdir = self.sym_mdl.getObject('Creo.Files.3DModel').getData()
        dir_mdl = os.path.join(self.sym_mdl.getAllFilesDirectory(), '_3DModel')
        fu.copyTree(sdir, dir_mdl)

        ''' Add 3DModel to component '''
        model3D = SymDirectory('3DModel')
        self.sym_mdl.getObject('Files').addParam(model3D)
        model3D.setData(dir_mdl)

        ''' Replace parameters '''
        for par in self.sym_mdl['Creo.Replace'].getParam():
            lstpar = self.parCAD.getParameters(type(par).__name__)
            try:
                idx = [i.getName() for i in lstpar].index(par.getName())
                container = lstpar[idx].getParent()
                # Find position of parameter
                pos = [i.getName() for i in container.getParam()].index(par.getName())
                lstpar[idx].removeParam()
                container.addParam(par, pos)
            except:
                print(par.getName() + ' not found')

        if self.sym_mdl.getObject('Project.Stator.Slot'):
            segment_part = self.sym_mdl.getObject('Project.Stator.Slot')
        elif self.sym_mdl.getObject('Project.Rotor.Segment'):
            segment_part = self.sym_mdl.getObject('Project.Rotor.Segment')
        else:
            raise ValueError('Segment part not found')

        par_part = segment_part['Part']
        par_part.setData(os.path.join(dir_mdl, os.path.basename(par_part.getData())))
        for par in segment_part['Settings'].getParameters('SymFile'):
            par.setData(os.path.join(dir_mdl, os.path.basename(par.getData())))

        self.sym_mdl.regenerate()

